{% extends "base.html" %}

{% block content %}
    <div class="container">

        <div class="jumbotron">
            <h1>{{ project }}</h1>
            <ul class="list-group">
                {% for version, location in versions.iteritems() %}
                    <li class="list-group-item"><a href="{{ location }}">{{ version }}</a></br></li>
                {% endfor %}
            </ul>

        </div>

    </div>
{% endblock %}
