{% extends "base.html" %}

{% block content %}
<div class="container">

      <div class="jumbotron">
        <h1>Cheddar</h1>

        <ul class="list-group">
          {% for distribution in history %}
          <li class="list-group-item"><a href="/simple/{{ distribution }}">{{ distribution }}</a></li>
          {% endfor %}
        </ul>

      </div>

    </div>
{% endblock %}
