{% extends "base.html" %}

{% block content %}
    <div class="container">

        <div class="jumbotron">
            <h1>Hosted Projects</h1>
            <ul class="list-group">
                {% for project in projects %}
                    <li class="list-group-item"><a href="/simple/{{ project }}">{{ project }}</a></li>
                {% endfor %}
            </ul>
        </div>

    </div>
{% endblock %}
